
#include <string>
#include <iostream>

//#include "iopclib/iopc.h"
//#include "iopclib/databaseObject.h"
#include "testClass.h"
//#include "testClass2.h"
//#include "iopcmeta/iopcMetaHeaders.h"
#include "iopclib/iopcHeaders.h"
//using namespace iopc;
using namespace std;
using namespace iopc;


int main(int argc, char *argv[])
{
	Iopc::init();
	LogWriter::getInstance().traceMaskTTY = ( LogWriter::WARNING | LogWriter::ERROR | LogWriter::INFO_SQL /*| LogWriter::TRACE | LogWriter::INFO*/);

	cout << "Start" << endl;

	const map<string, const Type*>& types = TypeManager::getTypesMap();
	cout << "Registered types:" << endl;
	for(map<string, const Type*>::const_iterator it = types.begin(); it != types.end(); it++) {
		cout << (*it).first << endl;
	}


	Person* pers = (Person*) TypeDesc<Person>::getType().newInstance();
	*((EString*)pers->getAttributeValue("name").getValue()) = "John Smith";
	*((EInt*)pers->getAttributeValue("age").getValue()) = 28;
	pers->write();

	Student stud("Janet White", 29, "MFF UK");
	const Type::ParentsList& parents = stud.getType().getParents();
	cout << "Parents: " << endl;
	for(Type::ParentsIterator it = parents.begin(); it != parents.end(); it++) {
		cout << (*it)->getQualifiedName() << endl;
	}
	cout << "Attributes: " << endl;
	const Object::AttributeValuesMap& attrVals = stud.getAttributeValues();
	for(Object::AttributeValuesIterator it = attrVals.begin(); it != attrVals.end(); it++) {
		const AttributeValue& val = it->second;
		cout << val.getAttribute().getType().getName() << " " <<  it->first << endl;
		if (val.getAttribute().getType().getDataTypeClass() == Type::IOPC_TYPECLASS_ENHANCED) {
			cout << ": " << ((EnhancedTypeBase*)val.getValue())->toString();
		}
		cout << endl;
	}

	// DB

	Database* db = DriverManager::getDriver("IopcOracle10g").getDatabase("XE");
	Connection* conn = db->getConnection("username=iopc;password=iopc");
	conn->open();
	conn->setAutoCommit(false);
	try {
		conn->sqlNonQuery("DROP TABLE iopc_test");
	} catch (...) {}
	conn->sqlNonQuery("CREATE TABLE iopc_test(id NUMBER PRIMARY KEY, name VARCHAR2(20))");
	conn->sqlNonQuery("INSERT INTO iopc_test VALUES(1, 'John Smith')");
	conn->sqlNonQuery("INSERT INTO iopc_test VALUES(2, 'Janet White')");
	conn->commit();
	EString name;
	EInt id;
	Cursor* cur = conn->sql("SELECT id, name from iopc_test");
	cur->addEnhancedOutParam(1, id);
	cur->addEnhancedOutParam(2, name);
	cur->execute();
	while (cur->fetchNext()) {
		cout << id << " " << name << endl;
	}
	cur->close();
	conn->returnCursor(cur);

	cur = conn->sql("SELECT name from iopc_test where id = :id");
	id = 2;
	cur->addEnhancedOutParam(1, name);
	cur->addEnhancedInParam(1, id);
	cur->execute();
	while (cur->fetchNext()) {
		cout << id << " " << name << endl;
	}
	cur->close();


	vector<string> script;

	// drop script
	script = ScriptsGenerator::getDbDropScript(DriverManager::getDriver("IopcOracle10g"));

	for(vector<string>::const_iterator it = script.begin(); it != script.end(); it++) {
		//LogWriter::logInfo(*it);
		try {
			conn->sqlNonQuery(*it);
		} catch (std::exception& e) {
			LogWriter::logInfo(e.what());
		}
	}

	// create script
	script = ScriptsGenerator::getDbCreateScript(DriverManager::getDriver("IopcOracle10g"));

	for(vector<string>::const_iterator it = script.begin(); it != script.end(); it++) {
		conn->sqlNonQuery(*it);
		//LogWriter::logInfo(*it);
	}


	conn->returnCursor(cur);
	conn->close();
	db->returnConnection(conn);
	DriverManager::getDriver("IopcOracle10g").returnDatabase(db);
	cout << "end" << endl;
	Iopc::shutdown();
	cin.getline(NULL, 0);
	return 0;
}

/*

export LD_LIBRARY_PATH=../iopclib/Debug/


gccxml -I../common -I. -I../driver_oracle8i -I../iopclib -O0 -g3 -pedantic -pedantic-errors -Wall -Werror -fmessage-length=0 -Wno-non-template-friend main.cpp -fxml=main.cxml
Debug/iopcsp main.cxml main.cpp
gcc -I../common -I. -I../driver_oracle8i -I../iopclib -O0 -g3 -pedantic -pedantic-errors -Wall -Werror -c -fmessage-length=0 -Wno-non-template-friend main.ic.cpp
ar -r main.a main.o main.ic.o
gcc -liopclib -lclntsh -L../iopclib/Debug -L/home/oracle/OraHome_1/lib/ main.a -o main
*/
