#ifndef CONDVAR_H_
#define CONDVAR_H_

#include <pthread.h>

#include "defs.h"
#include "exceptionBase.h"
#include "timeStamp.h"

namespace iopc {

/** An encapsulated conditional - includes conditional proper and its respective mutex. */
class DLL_EXTERNAL CondVar {
public:
	CondVar()
		throw (IopcNoMemoryException, IopcRsrcLackException);
	~CondVar()
		throw ();
		
	/** Locks mutex. */
	void lock()
		throw (IopcRsrcLackException, IopcDeadlockException);
		
	/** Unlocks mutex. */
	void unlock()
		throw (IopcIncorrectUsageException);
		
	/** Send signal to sleepers. */
	void signal()
		throw ();
		
	/** Wait for signal. */
	void wait()
		throw ();
		
	/** 
	 * @brief Wait for signal for limited time.
	 * 
	 * @param ts Absolute time specifying sleep limit.
	 */
	void wait(const TimeStamp& ts)
		throw ();
		
private:
	CondVar(const CondVar&);
	//CondVar &operator=(const CondVar&);
	pthread_mutex_t mutex;	/**< Mutex. */
	pthread_cond_t cond;	/**< Conditional. */
};

}

#endif /*CONDVAR_H_*/
