#ifndef COUNTEDPTR_H_
#define COUNTEDPTR_H_

/*
 * CountedPtr - simple reference counted pointer.
 *
 * The is a non-intrusive implementation that allocates an additional
 * int and pointer for every counted object.
 */
namespace iopc {

template <class X> class CountedPtr
{
public:
    typedef X element_type;

    explicit CountedPtr(X* p = 0, bool allowDelete = true) // allocate a new counter
        : itsCounter(0) {if (p) itsCounter = new counter(p, 1, allowDelete);}
    /*explicit CountedPtr(X* p = 0, bool allowDelete = true)
        : itsCounter(0) {if (p) itsCounter = new counter(p, allowDelete);}*/
    ~CountedPtr()
        {release();}
    CountedPtr(const CountedPtr& r) throw()
        {acquire(r.itsCounter);}
    CountedPtr& operator=(const CountedPtr& r)
    {
        if (this != &r) {
            release();
            acquire(r.itsCounter);
        }
        return *this;
    }


    X& operator*()  const throw()   {return *itsCounter->ptr;}
    X* operator->() const throw()   {return itsCounter->ptr;}
    X* get()        const throw()   {return itsCounter ? itsCounter->ptr : 0;}
    bool unique()   const throw()
        {return (itsCounter ? itsCounter->count == 1 : true);}
    bool isNull()   const throw()   {return itsCounter == NULL; }
    void allowDelete(bool allow)  throw() {if (itsCounter) itsCounter->allowDelete = allow; }

private:

    struct counter {
        counter(X* p = 0, unsigned c = 1, bool a = true) : ptr(p), count(c), allowDelete(a) {}
        X*          ptr;
        unsigned    count;
        bool		allowDelete;
    }* itsCounter;

    void acquire(counter* c) throw()
    { // increment the count
        itsCounter = c;
        if (c) ++c->count;
    }

    void release()
    { // decrement the count, delete if it is 0
        if (itsCounter) {
            if (--itsCounter->count == 0) {
            	if (itsCounter->allowDelete) {
                	delete itsCounter->ptr;
            	}
                delete itsCounter;
            }
            itsCounter = 0;
        }
    }
};

}

#endif /*COUNTEDPTR_H_*/
