#ifndef LOCK_H_
#define LOCK_H_

/**
 * @file lock.h
 * 
 * Recursive mutex
 * 
 * @author Petr Cermak
 */

#include <pthread.h>
#include "exceptionBase.h"

namespace iopc {

/** An encapsulated mutex. */
class DLL_EXTERNAL Lock {
public:
	Lock()
		throw (IopcNoMemoryException, IopcRsrcLackException);
	~Lock()
		throw ();
	/** Locks mutex. */
	void lock()
		throw (IopcDeadlockException);
	/** Unlocks mutex. */
	void unlock()
		throw (IopcIncorrectUsageException);
private:	/* hiding */
	Lock(const Lock &);
	Lock &operator=(const Lock &);
protected:
	pthread_mutex_t mutex;	/**< Mutex proper. */
	pthread_mutexattr_t mutexattr; /**< Recursive attribute. */
};

}/*namespace iopc*/


#endif /*LOCK_H_*/
