#include "metadataHolder.h"
#include <string>
#include <sstream>

using namespace std;

namespace iopc {

MetadataHolder::MetadataHolder()
: secondaryMeta(NULL)
{
}

void MetadataHolder::setSecondaryMetadataLocation(const MetadataHolder* secondaryMeta)
{
	this->secondaryMeta = secondaryMeta;
}

MetadataHolder& MetadataHolder::getDefaultsMeta()
{
	static MetadataHolder defaults;
	return defaults;
}

const Metadata& MetadataHolder::getMeta(const std::string& key, bool searchDefaults, bool searchSecondary) const
{
	std::map<std::string, Metadata>::const_iterator it = meta.find(key);
	if (it == meta.end() || it->second.isEmpty()) {
		if (hasSecondaryMetadataLocation() && searchSecondary) {
			if (secondaryMeta->hasMeta(key, false, false)) {
				return secondaryMeta->getMeta(key, false, false);
			}
		}
		if (searchDefaults) {
			return getDefaultsMeta().getMeta(key, false, false);
		}
		IOPC_LOG_THROW(IopcIncorrectUsageException(string("No such metadata found: ") + key));
	}
	return it->second;
}

void MetadataHolder::setMeta(const std::string& key, const Metadata& value) const
{
	meta[key] = value;
}

bool MetadataHolder::hasMeta(const std::string& key, bool searchDefaults, bool searchSecondary) const
{
	std::map<std::string, Metadata>::const_iterator it = meta.find(key);
	if (it != meta.end()) { // || (it->second).isEmpty()) {
		return true;
	}
	if (hasSecondaryMetadataLocation() && searchSecondary) {
			if (secondaryMeta->hasMeta(key, false, false)) {
				return true;
			}
		}
	if (searchDefaults) {
		if (getDefaultsMeta().hasMeta(key, false, false)) {
			return true;
		}
	}
	return false;
}

string MetadataHolder::toString() const
{
	ostringstream str;
	str << *this;
	return str.str();
}

const Metadata& AdditionalMetadataHolder::getMeta(const string& key, bool searchDefaults, bool searchSecondary) const
{
	if (!invert) {
		if (primary.hasMeta(key)) {
			return primary.getMeta(key, searchDefaults, searchSecondary);
		}
		return MetadataHolder::getMeta(key, searchDefaults, searchSecondary);
	}
	
	if (MetadataHolder::hasMeta(key, searchDefaults, searchSecondary)) {
		return MetadataHolder::getMeta(key, searchDefaults, searchSecondary);
	} 
	return primary.getMeta(key, searchDefaults, searchSecondary);
}

bool AdditionalMetadataHolder::hasMeta(const std::string& key, bool searchDefaults, bool searchSecondary) const
{
	return MetadataHolder::hasMeta(key, searchDefaults, searchSecondary) || primary.hasMeta(key, searchDefaults, searchSecondary);
}

std::string Metadata::toString() const
{
	std::ostringstream out;
	switch (type) {
		case Metadata::INT:
			out << intValue;
			break;
		case Metadata::STRING:
			out << stringValue;
			break;
		case Metadata::BOOL:
			out << boolValue;
			break;
		default:
			out << "<other> (can't be serialized)";
			//IOPC_LOG_THROW(IopcIncorrectUsageException("This metadata type can't be serialized"));
			break;
	}
	return out.str();
}


std::ostream& operator<<(std::ostream& out, const Metadata& m)
{
	switch (m.type) {
		case Metadata::INT:
			out << "<int> ";
			out << m.intValue;
			break;
		case Metadata::STRING:
			out << "<string> ";
			out << m.stringValue;
			break;
		case Metadata::BOOL:
			out << "<bool> ";
			out << m.boolValue;
			break;
		default:
			out << "<other> (can't be serialized)";
			//IOPC_LOG_THROW(IopcIncorrectUsageException("This metadata type can't be serialized"));
			break;
	}
	return out;
}

std::ostream& operator<<(std::ostream& out, const MetadataHolder& m)
{
	out << "Metadata: " << endl;
	for (std::map<std::string, Metadata>::const_iterator it = m.meta.begin(); it != m.meta.end(); it++)
	{
		out << it->first << ": ";
		out << it->second;
		out << endl;
	}
	if (m.secondaryMeta != NULL) {
		out << "Secondary Metadata: " << endl;
		out << *m.secondaryMeta;
	}
	out << endl;
	return out;
}


MetadataHolder MetadataHolder::emptyMeta;

}

