/**
 * @file mutex.h
 * 
 * @author Jan Hadrava
 * @author Petr Cermak
 */


#ifndef IOPCMUTEX_H
#define IOPCMUTEX_H

#include <pthread.h>
#include "exceptionBase.h"

namespace iopc {

/** An encapsulated mutex. */
class DLL_EXTERNAL Mutex {
public:
	Mutex()
		throw (IopcNoMemoryException, IopcRsrcLackException);
	~Mutex()
		throw ();
	/** Locks mutex. */
	void lock()
		throw (IopcDeadlockException);
	/** Unlocks mutex. */
	void unlock()
		throw (IopcIncorrectUsageException);
private:	/* hiding */
	Mutex(const Mutex &);
	Mutex &operator=(const Mutex &);
protected:
	pthread_mutex_t mutex;	/**< Mutex proper. */
};

}/*namespace iopc*/


#endif
