/**
 * @file basicDatabase.h
 * 
 * @author Michal Kopecky
 * @author Jan Hadrava
 * @author Josef Troch
 * @author Petr Cermak
 *
 * @brief A part of a database abstraction layer
 */
#ifndef BASICDATABASE_H
#define BASICDATABASE_H

#include <fstream>
#include <set>
#include "iopccommon/defsPlatform.h"
#include "database.h"
#include "iopccommon/mutex.h"


namespace iopc {

/**
 * @brief Represents a database instance
 * 
 * Methods of this class allow user to create and manipulate connections
 * to this database.
 * 
 * This class is thread safe and can be used by more threads concurrently.
 *
 * @author Michal Kopecky
 * @author Petr Cermak
 */
class DLL_EXTERNAL BasicDatabase : public Database {
	
private:
	typedef std::set<Connection*> ConnectionSet;
	typedef ConnectionSet::iterator ConnectionSetIterator;
	//std::string connectionString;
	static ConnectionSet allConnections;
	ConnectionSet connections;
	
	void addConnection(Connection* con);
	void removeConnection(Connection* con);
	
	DatabaseImpl* implementation;
	
	Lock lock;
	static Lock allLock;
	Driver& driver;
	
	
public:
	BasicDatabase(DatabaseImpl* implementation, Driver& driver);
	virtual ~BasicDatabase();
	
	//virtual bool Assign(const char *ConnectString);
	virtual void commit();
	virtual void rollback();
	virtual Connection* getConnection(
	                             const std::string& connStr /* TODO az bude Cache ,
	                                   Cache *DefaultCache = NULL*/
	                                 );
	virtual void returnConnection(Connection* connection);
	// TODO: returnAllConnections();
	virtual void disconnect();
	
	//inline virtual std::string& getConnectionString() { return connectionString; }
	
	inline ConnectionSet getConnections() { return connections; }
	inline static ConnectionSet getAllConnections() { return allConnections; }
	
	inline DatabaseImpl* getImplementation() { return implementation; }

	inline Driver& getDriver() { return driver; }

	/* TODO pokud bude potreba od Trocha
	static bool runAfterConnectHandler;
      	If \ref afterConnectHandler returns 0, \ref runAfterConnectHandler should be set to false.
	static int (* afterConnectHandler)(Connection * connection);
	*/
	
	virtual Database* getBasicDatabase();
	
}; // class


} // namespace

#endif
