/**
 * @file driverManager.cpp
 *
 * @author Petr Cermak
 */

#include "driverManager.h"
#include "database.h"
#include <string>
//#include <iostream>

using namespace std;

namespace iopc {


DriverManager::DriverManager()
{
	//std::cout << "DriverManager()" << std::endl;
}


DriverManager::~DriverManager()
{
	//std::cout << "~DriverManager()" << std::endl;
	//drivers.clear();
}

DriverManager& DriverManager::getInstance()
{
	static DriverManager instance;
	return instance;
}

void DriverManager::registerDriver(Driver* driver, const std::string& name)
{
	mutex.lock();
	drivers[name] = driver;
	mutex.unlock();
}

void DriverManager::registerDriverExtension(DriverExtension* driverEx, const std::string& driverName, const std::string& name)
{
	mutex.lock();
	driverExtensions[driverName][name] = driverEx;
	mutex.unlock();
}

Driver& DriverManager::getDriverInst(const std::string& name)
{
	mutex.lock();
	map<string, Driver*>::iterator it = drivers.find(name);
	if (it == drivers.end()) {
		IOPC_LOG_THROW(IopcNotFoundException(string(" \"") + name + "\" driver not found."));
	}
	Driver& driver = *(it->second);
	mutex.unlock();
	return driver;
}

void DriverManager::initDrivers()
{
	mutex.lock();
	for(map<string, Driver*>::iterator it = drivers.begin(); it != drivers.end(); ++it) {
		it->second->init(driverExtensions[it->first]);
		/*if (driverExtensions.find(it->first) != driverExtensions.end()) {

		} else {
			it->second->init();
		}*/
	}
	mutex.unlock();
}

void DriverManager::shutdownDrivers()
{
	mutex.lock();
	for(map<string, Driver*>::iterator it = drivers.begin(); it != drivers.end(); ++it) {
		it->second->shutdown();
	}
	mutex.unlock();
}

}
