/**
 * @file oracle10gOci.h
 * 
 * @author Josef Troch Josef Troch (<a href="mailto:josef.troch@email.cz">josef.troch@email.cz</a>) 2003-2004
 * @author Petr Cermak
 */

#ifndef ORACLE10GOCI_H
#define ORACLE10GOCI_H

#ifdef _MSC_VER
#pragma warning( disable : 4127 4100 ) //conditional expression is constant, unreferenced formal parameter
#pragma warning( disable : 4514)       //unreferenced inline function has been removed
#endif

//#define text OraText //!< To remove problem with redefinition of 'text' (fstream.h x oratypes.h).

#include "oci.h"
#include "ociap.h"
#include <string>
#include <utility>
#include "iopccommon/trace.h"

//#undef text

namespace iopc {

class Type;
	
/** Is @p x "OK" return value for OCI call ? */
#define OCI_OK_EXT(x) ((x==OCI_SUCCESS) || (x==OCI_SUCCESS_WITH_INFO) || (x==OCI_NO_DATA)) 
/** Is @p x simple "OK" return value for OCI call ("OK" and no additional information present - typical case). */
#define OCI_OK(x) ((x==OCI_SUCCESS))     
#define OCI_ERR(x) (!OCI_OK_EXT(x))
#define OCI_MSG(x) (!OCI_OK(x))


 /** According to type of @p ociRetVal creates error message

    @param ociEnvironment  Pointer to OCI environment handle.
    @param errorHandle     Pointer to OCI error handle.
    @param ociRetVal       Return value of OCI function to handle its (possible) error.
    @param funcName        Name of the OCI function.
    @param useEnvHandle    Set if OCIErrorGet() is to be called on @p ociEnvironment instead of @p errorHandle.

    @retval Error message and error code
   */
const std::pair<const std::string, const sb4> getOCIError(
	OCIEnv* ociEnvironment,
	OCIError* errorHandle,
	sword ociRetVal,
	const std::string& funcName,
	bool useEnvHandle);
	
inline const std::string getOCIMsg(
	OCIEnv* ociEnvironment,
	OCIError* errorHandle,
	sword ociRetVal,
	const std::string& funcName,
	bool useEnvHandle)
{
	const std::pair<const std::string, const sb4> p = getOCIError(ociEnvironment, errorHandle, ociRetVal, funcName, useEnvHandle);
	return p.first;
}

//bool testik();

#define IOPC_HANDLE_OCI_ERROR(ret, fname)							\
	do {															\
		if (OCI_ERR(ret)) {											\
			IOPC_LOG_THROW(IopcDbException(getOCIMsg(ret, fname)));	\
		} else if (OCI_MSG(ret)) {									\
			LogWriter::logInfo(getOCIMsg(ret, fname));			\
		}															\
	} while (0)
          
          
/*! One of possible return values of getOracleType(): type unknown - cannot bind it 
    as Oracle external data type.
 */
#define UNKNOWN_DATA_TYPE 0 
          
//ub2 getOracleType(const Type& varType);


}
                          
#endif /*ORACLE10GOCI_H*/
