#include "oracle10gORExtension.h"
#include "iopcmeta/typeManager.h"
#include "iopclib/oidObject.h"
#include "iopcdb/driver.h"
#include "iopclib/dbPtrBase.h"
#include <string>

using namespace iopc;
using namespace std;


Oracle10gORExtension::Oracle10gORExtension(const std::string& driverName, const std::string& name)
	: DriverExtension(driverName, name), orStatementsFeature(cache), mappingStatementsFeature(cache), objectStatementsFeature(cache)
{
}

void Oracle10gORExtension::doInit(Driver& driver)
{
	MetadataHolder& defaults = MetadataHolder::getDefaultsMeta();
	defaults["db.oracle.useOracleObjects"].setBoolValue(false);
	
	orStatementsFeature.init(driver.getFeature<SqlStatementsFeature>(), driver.getFeature<TypeMappingFeature>());
	objectStatementsFeature.init(driver.getFeature<SqlStatementsFeature>(), driver.getFeature<TypeMappingFeature>());
	mappingStatementsFeature.init(
		driver.getFeature<SqlStatementsFeature>(), 
		driver.getFeature<TypeMappingFeature>(),
		orStatementsFeature,
		objectStatementsFeature
	);
	driver.registerFeature<ORStatementsFeature>(orStatementsFeature);
	driver.registerFeature<ObjectStatementsFeature>(objectStatementsFeature);
	driver.registerFeature<MappingStatementsFeature>(mappingStatementsFeature);
	
	//processType(&TypeDesc<OidObject>::getType(), false, NULL);
	
	if (!TypeDesc<DbPtrBase>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<DbPtrBase>::getType()["db.oracle.type.sql"].setStringValue(
			TypeDesc<OidType>::getType().getMeta("db.oracle.type.sql").getStringValue()
			);
		TypeDesc<DbPtrBase>::getType()["db.oracle.type.sql.param"].setStringValue(
			TypeDesc<OidType>::getType().getMeta("db.oracle.type.sql.param").getStringValue());
	}
}
/*
void Oracle10gORExtension::processType(const Type* type, bool useOracleObjects, const Type* rootType)
{
	if (useOracleObjects) {
		if (!type->hasFeature("db.oracle.useOracleObjects")) {
			type->setFeature("db.oracle.useOracleObjects");
		}
		//(*type)["db.oracle.oracleObjects.root"].setPointerValue((void*)rootType);
	} else if (type->hasFeature("db.oracle.useOracleObjects")) {
		rootType = type;
		useOracleObjects = true;
		//(*type)["db.oracle.oracleObjects.root"].setPointerValue((void*)rootType);
	}
	
	if (useOracleObjects) {
		if (type->getParents().size() != 1) {
			IOPC_LOG_THROW(IopcNotSupportedException("Multiple inheritance is not supported if using oracle objects mapping."));
		}
		if (type->getMeta("db.mapping.type").getStringValue() != "vertical") {
			IOPC_LOG_THROW(IopcMetadataException("db.mapping.type", type->getQualifiedName(), "db.mapping.type must be 'vertical' if the db.oracle.useOracleObjects is true")); 
		}
		(*type)["db.oracle.oracleObjects.root"].setPointerValue((void*)rootType);
		if (!type->hasMeta("db.oracle.oracleObjects.type")) {
			(*type)["db.oracle.oracleObjects.type"].setStringValue(string("t") + type->getName());
		}
	}
		
	for (Type::ChildrenIterator it = type->getChildren().begin(); it != type->getChildren().end(); it++) {
		const Type* child = *it;
		processType(child, useOracleObjects, rootType);
	}
}
*/

static Oracle10gORExtension oracle10gExtensionInstance("IopcOracle10g", "IopcOracle10gORExtension");

