#ifndef VOIDCACHEITEM_H_
#define VOIDCACHEITEM_H_

#include "iopccommon/defs.h"
#include "cache.h"
#include "../databaseObject.h"
#include "iopccommon/rwxLock.h"

namespace iopc {

class VoidCache;

/** Represents an object in Void Cache */
class DLL_EXTERNAL VoidCacheItem : public CacheLock {
public:

	/**
	 * @param cache Owner of the item being created.
	 */
	VoidCacheItem(VoidCache& cache, const PersIdentification& persId, DatabaseObject& obj, bool isNew);
	virtual ~VoidCacheItem();
	virtual DatabaseObject& getObject();
	virtual CacheLock& addLock(bool rdonly);
	virtual void rmLock();
	
	virtual bool isItemLocked();
	
	/**
	 * @brief Unlocks the item, but doesn't remove it from the cache 
	 * 
	 * Don't call this method directly. Used by the VoidCache.
	 */
	virtual void unlock();	// for VoidCache
	
	inline bool isNew() { return isNewItem; };
	
private:
	VoidCacheItem();
	VoidCacheItem(const VoidCacheItem&);
	VoidCacheItem &operator=(const VoidCacheItem&);

	bool isNewItem;	/**< Indicates whether the item represents an object which doesn't have persistent copy yet. */
	VoidCache& cache;		/**< Owner of the item. */
	PersIdentification persId;	/**< ID of the object being represented by the item. */
	DatabaseObject* objectPtr;		/**< The object being represented by the item. */
	RWXLock lock;
};

}

#endif /*VOIDCACHEITEM_H_*/
