#include "dbPtrBase.h"
#include "iopccommon/trace.h"

using namespace std;

namespace iopc {

// TODO: vyjimky z newInstance??
DbPtrBase::DbPtrBase(const Type& type/*, bool createInstance*/)
	throw ()
: conn(NULL), persId(type)
{
	//IOPC_FUNC_INVOKED;
	state = TRANSIENT;
	setNull();
}

/*
DbPtrBase::DbPtrBase(const Type& type, CachedConnection* conn)
	throw ()
: objectPtr(NULL), conn(conn), persId(type)
{
	state = TRANSIENT;
}
*/

DbPtrBase::DbPtrBase(const Type& type, CachedConnection* conn, const OidType& oid, bool loadRealType)
	throw ()
: conn(conn), persId(type, oid, loadRealType)
{
	IOPC_FUNC_INVOKED;
	state = PERSISTENT;
	resetNull();
	/*if (virtualize) {
		persId.setSuperType(*conn);
		//persId.proto = &persId.proto->RealPrototype(conn, persId.keyValues);
	}*/
}

DbPtrBase::DbPtrBase(const Type& type, CachedConnection* conn, const KeyValues& keyValues)
	throw ()
: conn(conn), persId(type, keyValues)
{
	IOPC_FUNC_INVOKED;
	state = PERSISTENT;
	resetNull();
	/*if (virtualize) {
		persId.setSuperType(*conn);
		//persId.proto = &persId.proto->RealPrototype(conn, persId.keyValues);
	}*/
}

DbPtrBase::DbPtrBase(const DbPtrBase& src)
	throw ()
: conn(src.conn), persId(src.persId)
{
	IOPC_FUNC_INVOKED;
	state = src.getState();
	setNull(src.isNull());
	/*if (src.getState() == PERSISTENT) {
		state = PER
	} else {
		createTransientCopyOf(src);
	}*/
}

DbPtrBase::~DbPtrBase()
	throw ()
{
	/*IOPC_FUNC_INVOKED;
	if (isValid() && getState() == TRANSIENT) {
		setObjectPtrNull(); // deletes object
		//deleteObject();
	}*/
}

CacheLock& DbPtrBase::bePersistent(CachedConnection* conn)
{
	IOPC_FUNC_INVOKED;
	if (isPersistent()) {
		IOPC_LOG_THROW(IopcInvalidStateException("Object is already persistent"));
	}
	if (isObjectPtrNull()) {
		IOPC_LOG_THROW(IopcInvalidStateException("Object is not set"));
	}
	if (!isObjectPtrUnique()) {
		IOPC_LOG_THROW(IopcInvalidStateException("There are more references to the transient object. Copied DbPtrs, retreived CachePtrs etc. Because the cache is taking over this object no more references are allowed."));
	}

	try {
		//objectPtr->bePersPrepare(*conn);
		persId.initIdentity(*conn->getInner(), getObject());
		//objectPtr->setPersistent(*conn);
		CacheLock& cLock = conn->makePersistent(persId, getObject());
		this->conn = conn;

		//objectPtr.allowDelete(false);  // The pointer is not owned by the CountedPtr anymore
		setObjectPtrNull(false);  // The object is now owned by the cache
		resetNull();
		state = PERSISTENT;
		return cLock;
	}
	IOPC_LOG_CATCH(std::exception& e)
		persId.invalidate();
		//getObject().setTransient();
		IOPC_LOG_RETHROW(e);
	IOPC_LOG_CAUGHT;
}

void DbPtrBase::replaceWith(const DbPtrBase& src)
{
	// Reference counting will handle this
	/*if (getState() == TRANSIENT) {
		deleteObject();
	}*/
	state = src.getState();
	conn = src.conn;
	persId = src.persId;
	setNull(src.isNull());
	setDirty();
	//conn(src.conn), persId(src.persId),
}

void DbPtrBase::deleteObject()
{
	IOPC_FUNC_INVOKED;
	IOPC_ASSERT(isValid() && !isPersistent());
	//objectPtr->freeAll();
	//delete objectPtr;
	setObjectPtrNull();
	setNull();
}


DbPtrBase::operator bool() const
	throw ()
{
	IOPC_FUNC_INVOKED;
	return isValid();
}

void DbPtrBase::dbDelete()
	throw (IopcIncorrectUsageException)
{
	IOPC_FUNC_INVOKED;
	/* an exception can be thrown here */
	if (!isPersistent())
		IOPC_LOG_THROW(IopcIncorrectUsageException("Not a persistent object."));

	conn->dbDelete(persId);
	// TODO: nacist objectptr?
	//state = TRANSIENT;
	setNull();
	//objectPtr = NULL;
}

void DbPtrBase::cacheDelete() const
	throw (IopcIncorrectUsageException)
{
	IOPC_FUNC_INVOKED;
	/* an exception can be thrown here */
	if (!isPersistent())
		IOPC_LOG_THROW(IopcIncorrectUsageException());
	conn->cacheDelete(persId);
}

/*void DbPtrBase::setPersId()
	throw (IopcIncorrectUsageException)
{
	//ASSERT(persId.proto != NULL);
	ASSERT(isValid() && !isPersistent());
	ASSERT(objectPtr != NULL);
	persId.keyValues = persId.proto->KeyValues(*objectPtr);
}*/

/*char *DbPtrBase::SelectKeyValues() const
	throw ()
{
	if (IsPersistent()) {
		return StrCpy(persId.keyValues);
	} else {
		ASSERT(objectPtr != NULL);
		return persId.proto->KeyValues(*objectPtr);
	}
}*/

CachedConnection& DbPtrBase::getConnection() const
		throw (IopcInvalidStateException)
{
	if (conn == NULL) {
		IOPC_LOG_THROW(IopcInvalidStateException("Connection hasn't been assigned yet"));
	}
	return *conn;
}

/*void DbPtrBase::setNewInstance(DatabaseObject* obj)
{
	objectPtr = obj;
	objectPtr->onIopcCreate();
}*/


void* DbPtrBase::externalizeCopy(unsigned int maxlength)
{
	if (state != PERSISTENT) {
		IOPC_LOG_THROW(IopcInvalidStateException("DbPtr has to have an Oid assigned. Call bePersistent first."));
	}
	return persId.getOidRef().externalizeCopy(maxlength);
	//return &oidCopy;
}

void DbPtrBase::updateFromExternalized()
{
	persId.getOidRef().updateFromExternalized();
	persId.setValid();
	resetNull();
	state = PERSISTENT;
	setDirty();
	setObjectPtrNull(true); // release the initially created object
}

const Type& DbPtrBase::getInnerType() const
{
	return persId.getConstOidRef().getInnerType();
	//return TypeDesc<OidTypeInner>::getType();
}

void* DbPtrBase::externalizeEmptyBuffer(unsigned int length)
{
	// do nothing. the buffer is allocated statically
	return persId.getOidRef().externalizeEmptyBuffer(length);
}

unsigned int DbPtrBase::getBufferSize()
{
	return persId.getOidRef().getBufferSize();
}
/*
bool DbPtrBase::updateValue(OidType newVal)
{

	if (persId.getOid() == newVal) {
		return false;
	}
	resetNull();
	persId = PersIdentification(persId.getType(), newVal);
	//value = newVal;
	setDirty();
	return true;
}
*/



const Type& DbPtrBase::getTypeInternal() const
{
	return TypeDesc<DbPtrBase>::getType();
	//return TypeManager::getTypeByTypeInfo(&typeid(*this));
}


// Todo: jde to asi udelat lepe zaregistrovanim callbacku do PersID.
void DbPtrBase::propagateConnectionToAttributes(DatabaseObject& o) const
{
	if (o.initializedAfterLoad) return;
// setconn dbptr atributum
	for(Object::AttributeValuesIterator it = o.getAttributeValues().begin(); it != o.getAttributeValues().end(); it++) {
		const AttributeValue& attrVal = it->second;
		if (attrVal.getAttribute().getType().is<DbPtrBase>() && !attrVal.getAttribute().hasFeature("db.transient")) {
			(dynamic_cast<DbPtrBase*>((Object*)attrVal.getValue()))->setConnection(conn);
		}
	}
	o.initializedAfterLoad = true;
}










TypeDesc<DbPtrBase> TypeDesc<DbPtrBase>::instance;



}/*namespace polite*/

