#ifndef KEYVALUES_H_
#define KEYVALUES_H_

#include "iopccommon/defsPlatform.h"
#include "iopcmeta/enhancedTypeBase.h"
#include "iopcmeta/enhancedNumber.h"
#include "iopcmeta/enhancedString.h"
#include <string>
#include <map>

namespace iopc {


class DLL_EXTERNAL KeyValues
{
private:
	std::map<std::string, EnhancedTypeBase*> values;

	void copyFrom(const KeyValues& src);
public:
	KeyValues();
	KeyValues(const KeyValues& src);


	typedef std::map<std::string, EnhancedTypeBase*>::const_iterator Iterator;

	inline Iterator begin() const {
		return values.begin();
	}

	inline Iterator end() const {
		return values.end();
	}

	inline int size() const {
		return values.size();
	}

	template<typename T>
	void add(std::string key, ENumber<T> value) {
		add(key, (EnhancedTypeBase&) value);
	}

	template<typename T>
	void add(std::string key, EStr<T> value) {
		add(key, (EnhancedTypeBase&) value);
	}

	template<typename T>
	void add(std::string key, T value);

	void add(std::string key, EnhancedTypeBase& value);
	void add(std::string key, EnhancedTypeBase* value);

	void addNoCopy(std::string key, EnhancedTypeBase* value);

	EnhancedTypeBase& get(const std::string& key) const;
	bool contains(const std::string& key) const;
	void clear();

	virtual ~KeyValues();

	KeyValues& operator=(const KeyValues& src);
	bool operator<(const KeyValues& id) const;	/* needed by STL containers */
	bool operator==(const KeyValues& id) const;
	bool operator!=(const KeyValues& id) const;

	int compare(const KeyValues& id) const;

	std::string toString() const;
};



}


#endif /*KEYVALUES_H_*/

