#include "query.h"
#include "iopccommon/util.h"
#include "iopcmeta/typeManager.h"
#include "oidObject.h"
#include "databaseObject.h"
#include <vector>

using namespace std;
using namespace iopc;

string Query::substituteQuery(string query, const Type& type, bool useSV) const
{
	vector<string> tokens;
	IOPC_DEBUGLOG(query + TYPE_DELIMITER);
	StringUtil::splitString(query, TYPE_DELIMITER, tokens);
	IOPC_DEBUGLOG(StringUtil::joinStrings(tokens, "@"));

	//vector<string> outTokens;
	int len = tokens.size();
	int i = 1;
	bool subst = false;
	for(vector<string>::iterator it = tokens.begin(); it != tokens.end(); it++, i++, subst = !subst) {
		if (!subst) {
			//outTokens.push_back(*it);
			continue;
		}
		if (subst && i == len) {
			continue;
		}

		if (*it == "") {
			*it = TYPE_DELIMITER;
			//outTokens.push_back(typeDelimiter);
		} else {
			// namespace::Type::attribute or just namespace::Type ?
			string attr;
			string typeName;
			string typeMeta;
			string attrMeta;
			bool fromTable = false;
			//string str = *it;
			TypeManager::parseType(*it, typeName, typeMeta, attr, attrMeta);

			if (!TypeManager::hasType(typeName)) {
				continue;
			}
			const Type& substType = TypeManager::getType(typeName);

			if (typeMeta != "" && type.hasMeta(typeMeta)) {
				*it = type.getMeta(typeMeta).toString();
			} else {
				if (typeMeta == "db.query.skip") {
					*it = "";
				} else if (substType.isTypeOf<OidObject>()) {
					if (useSV && substType.hasMeta("db.view.sv")) {
						*it = substType.getMeta("db.view.sv").getStringValue();
					} else if (substType.hasMeta("db.view.pv")) {
						*it = substType.getMeta("db.view.pv").getStringValue();
					}
				} else if (substType.isTypeOf<DatabaseObject>()) {
					*it = substType.getMeta("db.table").getStringValue();
					fromTable = true;
				} else {
					continue;
				}
			}
			if (attr != "") {
				if (substType.hasAttributeAll(attr)) {
					const Attribute& attribute = substType.getAttributeAll(attr);
					if (attrMeta != "") {
						*it += "." + attribute.getMeta(attrMeta).toString();
					} else {
						if (fromTable && attribute.hasMeta("db.column")) {
							*it += "." + attribute.getMeta("db.column").getStringValue();
						} else if (!fromTable && attribute.hasMeta("db.viewColumn")) {
							*it += "." + attribute.getMeta("db.viewColumn").getStringValue();
						} else {
							*it += "." + attr;
						}
					}
				} else {
					*it += "." + attr;
				}
			}
		}
	}

	return StringUtil::joinStrings(tokens, "");
}



