#ifndef RESULT_H_
#define RESULT_H_

#include "resultBase.h"
#include "iopcmeta/typeDesc.h"
#include "simpleQuery.h"
#include "resultIterator.h"


namespace iopc {

template<typename T> class ResultIterator;

/**
 * @brief Represents a unique identifier of a persistent object.
 */

class SimpleQuery;

template<typename T>
class DLL_EXTERNAL Result : public ResultBase {

	friend class ResultIterator<T>;
private:
	DbPtr<T> currentObject;

	DbPtr<T> get(int i) {
		return DbPtr<T>(conn, getKeyAt(i));
	}

	DbPtr<T> getCurrent() {
		if (isTransient) {
			return currentObject;
		} else {
			return DbPtr<T>(conn, getCurrentKey());
		}
	}

	//~ResultBase();

protected:
	virtual bool loadCurrentObject(Cursor& cur) {
		currentObject = DbPtr<T>(cur);
		return currentObject.isValid();
	}


public:
	typedef ResultIterator<T> iterator;

	Result(CachedConnection* conn, const Query& query, bool preload = true) : ResultBase(conn, query, preload, TypeDesc<T>::getType()) {};
	Result(CachedConnection* conn, bool preload = true) : ResultBase(conn, SimpleQuery::All, preload, TypeDesc<T>::getType()) {};

	iterator begin() { return iterator(true, preload ? getKeyCount() : -1, *this); }
	iterator end() { return iterator(false, preload ? getKeyCount() : -1, *this); }



};

}

#endif /*RESULT_H_*/
