#include "textFileMetadataLoader.h"
#include <iostream>
#include <fstream>
#include "iopccommon/trace.h"
#include "iopccommon/util.h"
#include "iopccommon/metadataHolder.h"
#include "iopccommon/globalSettings.h"
#include "typeManager.h"

using namespace iopc;
using namespace std;

void TextFileMetadataLoader::initializeMetadata()
{
	ifstream input(file.c_str(), ios::in);

	if (!input) {
		IOPC_LOG_THROW(IopcIOException(string("Error reading configuration file: ") + file));
	}

	string line;
	while (getline(input, line)) {
		vector<string> metadata;
		StringUtil::splitString(line, ";", metadata);
		if (line.length() == 0) continue;
		if (line[0] == '#') continue;
		if (metadata.size() != 3) continue;

		string datatype = metadata[1];
		string value = metadata[2];
		string type;
		string typeMeta;
		string attr;
		string attrMeta;

		TypeManager::parseType(metadata[0], type,  typeMeta,  attr, attrMeta);

		if (type == "defaults") {
			setMeta(MetadataHolder::getDefaultsMeta(), typeMeta, datatype, value);
			continue;
		}
		if (type == "globals") {
			setMeta(globalSettings, typeMeta, datatype, value);
			continue;
		}

		if (type == "" || !TypeManager::hasType(type)) {
			LogWriter::logWarning(string("Unknown type ") + type + " when loading from " + file);
			continue;
		}
		const Type& t = TypeManager::getType(type);
		if (typeMeta != "") {
			setMeta(t, typeMeta, datatype, value);
		} else if (attrMeta != "") {
			if (attr == "" || !t.hasAttribute(attr)) {
				LogWriter::logWarning(string("Unknown attribute ") + type + "::" + attr + " when loading from " + file);
				continue;
			}
			setMeta(t.getAttribute(attr), attrMeta, datatype, value);
		} else {
			LogWriter::logWarning(string("No metadata code on ") + type + " when loading from " + file);
		}
	}

	input.close();
}

void TextFileMetadataLoader::setMeta(const MetadataHolder& meta, string metadata, string type, string value)
{
	if (type == "string") {
		meta[metadata].setStringValue(value);
	} else if (type == "int") {
		meta[metadata].setIntValue(StringUtil::parseInt(value));
	} else if (type == "bool") {
		meta[metadata].setBoolValue(StringUtil::parseBool(value));
	} else {
		LogWriter::logWarning(string("Invalid metadata data type: ") + type);
	}
}


