#include "type.h"
#include "iopccommon/trace.h"
#include "typeManager.h"
#include <queue>

using namespace std;

namespace iopc {

Type::Type()
{
}

Type::~Type()
{
}

void Type::init()
{
	for (vector<const type_info*>::iterator i = parentsTypeInfo.begin(); i != parentsTypeInfo.end(); i++) {
		const Type* parent = &TypeManager::getTypeByTypeInfo(*i);
		parents.push_back(parent);
		const_cast<Type*>(parent)->children.push_back(this);
	}

	// TODO: odstranit - DEBUG
	this->typeName = getQualifiedName();
}

void Type::postInit()
{
	generateAllParents();
	generateAllChildren();
	generateInheritedAttributes();

	iopcInit();
}

void Type::generateAllParents()
{
	queue<const Type*> que;
	que.push(this);

	do {
		const Type* t = que.front();
		que.pop();
		for(Type::ParentsIterator it = t->getParents().begin(); it != t->getParents().end(); it++) {
			que.push(*it);
			allParents.insert(*it);
		}
	} while (!que.empty());
}

void Type::generateAllChildren()
{
	queue<const Type*> que;
	que.push(this);

	do {
		const Type* t = que.front();
		que.pop();
		for(Type::ChildrenIterator it = t->getChildren().begin(); it != t->getChildren().end(); it++) {
			que.push(*it);
			allChildren.insert(*it);
		}
	} while (!que.empty());
}


void Type::generateInheritedAttributes()
{
	for (AllParentsIterator it = allParents.begin(); it != allParents.end(); it++) {
		const AttributesMap& amap = (*it)->getAttributes();
		for (AttributesIterator it = amap.begin(); it != amap.end(); it++) {
			inheritedAttributes[it->first] = &it->second;
		}
		//inheritedAttributes.insert(amap.begin(), amap.end());
	}
}



const Type& Type::getFirstParent() const
{
	if (parents.size() > 0) {
		return *parents[0];
	}
	IOPC_LOG_THROW(IopcIncorrectUsageException("This type has no parent class."));
}

const Attribute& Type::getAttribute(const string& name) const
{
	map<string, Attribute>::const_iterator it = attributes.find(name);
	if(it == attributes.end()) {
		IOPC_LOG_THROW(IopcIncorrectUsageException(string("No ") + name + " attribute in the " + getQualifiedName() + " class."));
	}
	return (*it).second;
}

bool Type::hasAttribute(const string& name) const
{
	map<string, Attribute>::const_iterator it = attributes.find(name);
	return it != attributes.end();
}

bool Type::hasAttributeAll(const std::string& name) const
{
	AttributesIterator it = attributes.find(name);
	if(it == attributes.end()) {
		InheritedAttributesIterator it2 = inheritedAttributes.find(name);
		if (it2 == inheritedAttributes.end()) {
			return false;
		}
		return true;
	} else {
		return true;
	}
}

const Attribute& Type::getAttributeAll(const std::string& name) const
{
	AttributesIterator it = attributes.find(name);
	if(it == attributes.end()) {
		InheritedAttributesIterator it2 = inheritedAttributes.find(name);
		if (it2 == inheritedAttributes.end()) {
			IOPC_LOG_THROW(IopcIncorrectUsageException(string("No ") + name + " attribute in the " + getQualifiedName() + " class or its parents."));
		}
		return *(*it2).second;
	} else {
		return (*it).second;
	}
}

/*bool Type::hasMeta(const std::string& key) const
{
	map<string, string>::const_iterator it = meta.find(key);
	return it != meta.end();
}*/


bool Type::isTypeOf(const Type& type) const
{
	if (type == *this) {
		return true;
	}
	const AllParentsList& parents = getAllParents();
	AllParentsIterator it = parents.find(&type);
	return it != parents.end();
}


}
