#include "tests/typesTest.h"
#include "iopcmeta/typeManager.h"
#include "iopcmeta/typeDesc.h"
#include "testSchema/fakeObject.h"
#include "testSchema/diamond.h"

#include "iopclib/keyValues.h"
//#include "testSchema/fakeType.h"
#include <iostream>

using namespace iopc;
using namespace std;

TypesTest::TypesTest(const std::string& testName)
 : TestCase(testName)
{
}

TypesTest::~TypesTest()
{
}


bool TypesTest::doTest()
{
	bool ret = true;
	
	const map<string, const Type*>& types = TypeManager::getTypesMap();
	
	logMsg("Registered types");
	for(map<string, const Type*>::const_iterator it = types.begin(); it != types.end(); it++) {
		logMsg((*it).first);
	}
	
	// Simple type Test
	// The two ways of obtainig the type object
	const Type& type1 = TypeDesc<int>::getType();
	const Type& type2 = TypeManager::getType("::int");
	if (type1 != type2) {
		ret = false;
		logMsg("type1 != type2");
	}
	
	int* newInt = (int*) TypeManager::getType("::int").newInstance();
	*newInt = 5;
	delete newInt;
	
	// Complex type test
	
	FakeObject o;
	const Type& typeCmplx = TypeDesc<FakeObject>::getType();
	//cout << typeCmplx.getName() << "  " << o.getType().getName() << endl;
	if (typeCmplx.getName() != "FakeObject") {
		logMsg("getName != \"FakeObject\"");
		ret = false;
	}
	Object& obj = o;
	if (obj.getType() != TypeDesc<FakeObject>::getType()) {
		logMsg("Downcast failed");
		ret = false;
	}
	//cout << typeCmplx.getFirstParent().getQualifiedName() << endl;
	if (*typeCmplx.getParents().front() != TypeDesc<DatabaseObject>::getType()) {
		logMsg("typeCmplx.getParents().front() != TypeDesc<DatabaseObject>::getType()");
		ret = false;
	}
	
	// Dirty Test
	
	if (o.isDirty()) {
		logMsg("typeCmplx.isDirty == true");
		ret = false;
	}
	o.attrInt = 9;
	if (!o.isDirty()) {
		logMsg("typeCmplx.isDirty == false");
		ret = false;
	}
	if ((int)*(EInt*)o.getAttributeValue("attrInt").getValue() != 9) {
		logMsg("typeCmplx.getAttributeValue != ..");
		ret = false;
	}
	o.resetDirty();
	if (o.isDirty()) {
		logMsg("2. typeCmplx.isDirty == true");
		ret = false;
	}
	o.attrString = "ahoj";
	if (!o.isDirty()) {
		logMsg("2. typeCmplx.isDirty == false");
		ret = false;
	}
	
	// AllParents Test
	logMsg("All (direct and indirect) parents of DiamondClass4:");
	const Type& dt = TypeDesc<DiamondClass4>::getType();
	for (Type::AllParentsIterator it = dt.getAllParents().begin(); it != dt.getAllParents().end(); it++) {
		logMsg((*it)->getQualifiedName());
	}
	logMsg("All (direct and indirect) parents of DatabaseObject:");
	const Type& dtt = TypeDesc<DatabaseObject>::getType();
	for (Type::AllParentsIterator it = dtt.getAllParents().begin(); it != dtt.getAllParents().end(); it++) {
		logMsg((*it)->getQualifiedName());
	}
	
	
	// TypeOf Test
	if (! (dt.isTypeOf(TypeDesc<DiamondClass4>::getType()) &&
		dt.isTypeOf(TypeDesc<DiamondClass3>::getType()) &&
		dt.isTypeOf(TypeDesc<DiamondClass2>::getType()) &&
		dt.isTypeOf(TypeDesc<DiamondClass>::getType()) &&
		dt.isTypeOf(TypeDesc<Object>::getType()) )) {
		
		logMsg("IsTypeOf failed");
		ret = false;
	}
	
	KeyValues m;
	KeyValues n;
	m.add("a", EInt(5));
	m.add("b", EString("6d"));
	n.add("a", EInt(5));
	n.add("b", EString("6d"));
	
	if (m != n) {
		logMsg("Compare failed");
		ret = false;
	}
	
	return ret;
}

static TypesTest typesTestInstance("Type system Test");
